#include <Arduino.h>
#include <FSDisplay.h>
#if defined ( __XTENSA__ )
#include <ESP8266WiFi.h>
#endif

display::display() {
	oled=NULL;
	lcd=NULL;
}

display::display(U8X8_SH1106_128X64_NONAME_HW_I2C *oled_disp) {
	oled=oled_disp;
	lcd=NULL;
}

display::display(LiquidCrystal_I2C *lcd_disp) {
	lcd=lcd_disp;
	oled=NULL;
}

display::display(U8X8_SH1106_128X64_NONAME_HW_I2C *oled_disp, LiquidCrystal_I2C *lcd_disp) {
	oled=oled_disp;
	lcd=lcd_disp;
}

void display::begin() {
	timestamp=millis();
	if(oled) {
		oled->begin();
		oled->setPowerSave(0);
        	InitDisplay();
        	oled->setFont(showfont);
        	//and now display something about this device
        	oled->drawString(0,2,"Pellets Lager");
        	oled->drawString(0,4,"Fuellstand und");
        	oled->drawString(0,6,"Sensor Info");

	}
	if(lcd) {
		lcd->init();
		lcd->clear();
		lcd->backlight();
		lcd->setCursor(0,0);
		lcd->print(HEADLINE);
		lcd->setCursor(0,1);
		lcd->print("  Lager Infos   ");

	}
	delay(4000);
}

void display::InitDisplay() {
	if (oled) {
		//clear the display
		oled->clear();
		//set the bigger font for next textoutput
		oled->setFont(bigfont);
		//print the headline
		oled->drawString(0,0,HEADLINE);
	}
}

void display::ShowMask() {
	if (oled) {
        	InitDisplay();
        	//Create something like a "Display Mask"
        	oled->setFont(showfont);
		oled->drawString(0,DISTPOS,"Abst.: ");
		oled->drawString(0,LAGERPOS,"Pellets: ");
		oled->drawString(14,DISTPOS,"cm");
		oled->drawString(15,LAGERPOS,"t");
#if defined DHT11_SENSOR
		oled->drawString(0,LAGERPOS+2,"Temp: ");
		oled->drawString(15,LAGERPOS+2,"C");
#endif
	}
	if (lcd) {
	//Create something like a "Display Mask"
		lcd->setCursor(0,0);
		lcd->print("Abst.: ");
		lcd->setCursor(14,0);
		lcd->print("cm");
		lcd->setCursor(0,1);
		lcd->print("Pellets: ");
		lcd->setCursor(15,1);
		lcd->print("t");
	}
}
#if defined ( __XTENSA__ )
void display::NetConnect() {
	int pos=0,length;

        length=String(WiFi.SSID()).length();
	if(oled) {
        	InitDisplay();
		//get the length of the SSID Name
		//and calculate a centered position for the OLED Display
		pos=(16-length)/2;
		//and display the message, that the connection
		//will start now
		oled->setFont(showfont);
		oled->drawString(0,2,"Connecting WLAN");
		//show the SSID on OLED
		oled->drawString(pos,4,WiFi.SSID().c_str());
	}
	if(lcd) {
		lcd->setCursor(0,0);
		lcd->print("Connecting WLAN");
		lcd->setCursor(0,1);
		lcd->print("                ");
		lcd->setCursor(pos,1);
		lcd->print(WiFi.SSID().c_str());
	}
}
void display::ConnectOK() {
	if(oled) {
                //We have a WL_CONNECTED - wlan connection is setup
                oled->drawString(7,6,"OK");
	}
	if(lcd) {
                lcd->setCursor(0,1);
                lcd->print("                ");
                lcd->setCursor(7,1);
                lcd->print("OK");
	}
	delay(2000);
}
void display::ConnectFailed() {
	if(oled) {
                oled->drawString(6,6,"FAILED");
	}
	if(lcd) {
                lcd->setCursor(0,1);
                lcd->print("                ");
                lcd->setCursor(5,1);
                lcd->print("FAILED");
	}
	delay(2000);
}
#endif
void display::UpdateDisplay(FSSensor *sensor) {
        //and to position the string we need something
        //to calculate the correct pos.
        int pos=0;

        //The string should end on col 13 (of 16)
        //so we have to remove the length of the string
        //from the position where the string should end
        //this is than our position to display the value
#ifdef DEBUG
	pos=13-String(sensor->GetSens(0)).length();
#else
        pos=13-String(sensor->GetDistance()).length();
#endif
	if(oled) {
        	//let us setup the font used to display the values
        	oled->setFont(showfont);
        	//Now let us start with a clearing the place
        	//where to display the value
        	//this will avoid artefakts from old values
        	oled->drawString(6,DISTPOS,"       ");
        	//Now the same procedure as above for the second
        	//value.
#ifdef DEBUG
		oled->drawString(pos,DISTPOS,(const char *) String(sensor->GetSens(0)).c_str());
#else
        	oled->drawString(pos,DISTPOS,(const char *) String(sensor->GetDistance()).c_str());
#endif
	}
	if(lcd) {
        	lcd->setCursor(6,0);
        	lcd->print("        ");
        	lcd->setCursor(pos,0);
        	lcd->print(sensor->GetDistance());
	}
#ifdef DEBUG
	pos=14-String(sensor->GetSens(1)).length();
#else
        pos=14-String(sensor->GetPellets(),3).length();
#endif
	if(oled) {
		oled->drawString(8,LAGERPOS,"       ");
#ifdef DEBUG
		oled->drawString(pos,LAGERPOS,(const char *) String(sensor->GetSens(1)).c_str());
#else
		oled->drawString(pos,LAGERPOS,(const char *) String(sensor->GetPellets(),3).c_str());
#endif
	}
	if(lcd) {
		lcd->setCursor(8,1);
		lcd->print("      ");
		lcd->setCursor(pos,1);
		lcd->print(sensor->GetPellets());
	}
#if defined DHT11_SENSOR
	       pos=13-String(sensor->GetTemperature()).length();
	if(oled) {
	       	oled->drawString(8,LAGERPOS+2,"    ");
	       	oled->drawString(pos,LAGERPOS+2,(const char *) String(sensor->GetTemperature(),2).c_str());
	}
#endif
	timestamp=millis();
        //and give the ESP8266 time for wlan
        yield();
}
