#include <FSNetwork.h>
#if defined (__XTENSA__)

network::network() {
        void *ptr;
        ptr=(WiFiServer *) malloc(sizeof(WiFiServer));
        server = (WiFiServer *) new(ptr) WiFiServer(80);
}

void network::begin() {
        //wifimanager.resetSettings();
	wifimanager.setConfigPortalTimeout(300);
	if (! wifimanager.autoConnect("APSID") ) {
		ESP.restart();
	}
        ArduinoOTA.onStart([]() {
                String type;
                if (ArduinoOTA.getCommand() == U_FLASH) {
                        type = "sketch";
                } else { // U_FS
                        type = "filesystem";
                }
        });
        ArduinoOTA.begin();
	configTime(MY_TZ, MY_NTP_SERVER);
	//getNtpServer();
        server->begin();
        server->setNoDelay(true);
        yield();
}

void network::testNet() {
	int wifi_retry = 0;
	if( WiFi.status() != WL_CONNECTED ) {
		while(WiFi.status() != WL_CONNECTED && wifi_retry < 5 ) {
			WiFi.disconnect();
			wifimanager.setConfigPortalTimeout(300);
			wifimanager.autoConnect("APSID");
			wifi_retry++;
			delay(100);
		}
		server->begin();
		server->setNoDelay(true);
		yield();
	}
	if(wifi_retry >= 5) {
		ESP.restart();
	}
}

void network::handleWeb(FSSensor *sensor) {
        //some variables
        //here for the actual client connection
        WiFiClient client;
        //and a buffer used to build the
        //buffer to store the data send bei the client
        String header;

        //Let us show if some client
        //is available
        client = server->accept();
        if (client) {
                //do this only if we have a client
                String currentline = "";
                while(client.connected()) {
                        //and if the client connection was successfull
                        if (client.available()) {
                                //and the client is still there
                                //first of all we must take the initial GET call
                                //of the client anywhere (http protocoll definition)
                                //so let us read the send string char by char
                                char c = client.read();
                                header +=c;
                                if ( c == '\n' ) {
                                        //OK we recived the newline command
                                        if (currentline.length() == 0 ) {
                                                //and nothing else - it was just a
                                                //empty line -> we are ready to send our data

                                                //define a string to convert our
                                                //float values to stings
                                                String temp;
						String sec;
						//Let us construct the Webpage in a String
						temp="HTTP/1.1 200 OK\nContent-type:text/html\nConnection:close\n\n";
						temp+="<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n";
						temp+="<html><head><meta http-equiv=\"refresh\" content=\"";
						temp+=WEBREFRESH;
						temp+="\"></head><body><H1><center>\n";
						temp+=HEADLINE;
						temp+=" ";
						temp+=VERS;
						temp+="\n</center></H1><br><br>\n";
                                                temp+="<table border=0 cellpadding=\"8\" cellspacing=\"0\" width=\"100%\">\n";
						temp+="<tr><td>Zeitpunkt letzte Messung: </td><td>";
						temp+=twodigit(zeit.tm_mday);
						temp+=".";
						temp+=twodigit(zeit.tm_mon+1);
						temp+=".";
						temp+=zeit.tm_year + 1900;
						temp+=" ";
						temp+=twodigit(zeit.tm_hour);
						temp+=":";
						temp+=twodigit(zeit.tm_min);
						temp+=".";
						temp+=twodigit(zeit.tm_sec);
						temp+="\n</td></tr>\n";
                                                temp+="<tr><td>Fuellstand des Pelletlagers: </td><td>";
                                                //we want the pellts ammount in tonns with 3 values after .
                                                temp+=String(sensor->GetPellets(),3);
                                                temp+=" Tonnen\n</td></tr>\n";
                                                //now let us create the line for the
                                                //distance value (2 sign's after the .)
                                                temp+="<tr><td>SensorAbstand: </td><td>";
                                                temp+=String(sensor->GetDistance(),2);
                                                temp+=" cm\n</td></tr>\n";
#if defined DHT11_SENSOR
                                                //now let us create the line for the
                                                //distance value (2 sign's after the .)
                                                temp+="<tr><td>Temperatur: </td><td>";
                                                temp+=String(sensor->GetTemperature(),2);
                                                temp+=" C\n</td></tr>\n";
						temp+="<tr><td>Feuchte: </td><td>";
						temp+=String(sensor->GetHumidity(),2);
                                                temp+=" %\n</td></tr>\n";
#endif
						//generate the lines for the single sensors
						for (int i=0; i<=NUM_SENS-1; i++) {
							temp+="<tr><td>Sensor ";
							temp+=String(i+1);
							temp+=": </td><td>";
							temp+=String(sensor->GetSens(i),2);
							temp+=" cm\n</td></tr>\n";
						}
                                                //now some empty lines and the HTML closure
						temp+="</table></body></html>\n\n";
                                                //now let us send a web page
						client.println(temp);
                                                //and jump out -> we are ready and will close the connection
                                                break;
                                        } else {
                                                //this was not a empty line - so we have still to get data
                                                //empty the variable, and startover for the next line
                                                currentline = "";
                                        }
                                } else if ( c != '\r' ) {
                                        //collect the line send by client char for char.
                                        currentline += c;
                                }
                        }
                }
                //reset our header variable
                header = "";
                // and stop the client connection
                client.flush();
                client.stop();
        }
        //give ESP8266 time to to wlan stuff
        yield();
}

void network::handleOTA() {
        ArduinoOTA.handle();
}

void network::StoreTimeLastRead() {
	time_t now;

	now = time(nullptr);
	localtime_r(&now, &zeit);
}

String network::twodigit(int number) {
	String buffer="";

	if(number < 10) {
		buffer+="0";
	}
	buffer+=number;
	return(buffer);
}
#endif
