#include <FSSensor.h>
#include <config.h>

FSSensor::FSSensor() {
       void *ptr;

#if defined ( __XTENSA__ )
#if defined SENS_VL53L0X
	for ( int i=0; i<=NUM_SENS-1; i++) {
		ptr=malloc(sizeof(VL53L0X));
		tof_sens[i]=(VL53L0X *) new(ptr) VL53L0X();
	}
#endif
#endif
	temp = 20.0;
	timestamp=millis();
	distance = 0;
	pellets = 0;
	
#if defined ( __XTENSA__ ) 
#if defined DHT11_SENSOR
	ptr=malloc(sizeof(DHT));
	dht=(DHT *) new(ptr) DHT(DHT11_SENSOR,DHT11);
#endif
#endif
};

#if defined DHT11_SENSOR
FSSensor::FSSensor(DHT *dht11) {
#if defined SENS_VL53L0X
	void *ptr;
	for ( int i=0; i<=NUM_SENS-1; i++) {
		ptr=malloc(sizeof(VL53L0X));
		tof_sens[i]=(VL53L0X *) new(ptr) VL53L0X();
	}
#endif
	temp = 20.0;
	timestamp=millis();
	distance = 0;
	pellets = 0;
	dht = dht11;
}
#endif

#if defined HCSR04
FSSensor::FSSensor(int Echo1, int Trig1) {
	temp = 20.0;
	timestamp=millis();
	distance = 0;
	pellets = 0;
	EchoPin[0]=Echo1;
	TrigPin[0]=Trig1;
	EchoPin[1]=1000;
	TrigPin[1]=1000;
}

#if defined DHT11_SENSOR
FSSensor::FSSensor(int Echo1, int Trig1, DHT *dht11) {
	temp = 20.0;
	timestamp=millis();
	distance = 0;
	pellets = 0;
	dht=dht11;
	EchoPin[0]=Echo1;
	TrigPin[0]=Trig1;
	EchoPin[1]=1000;
	TrigPin[1]=1000;
}
#endif

FSSensor::FSSensor(int Echo1, int Trig1, int Echo2, int Trig2) {
	temp = 20.0;
	timestamp=millis();
	distance = 0;
	pellets = 0;
	EchoPin[0]=Echo1;
	TrigPin[0]=Trig1;
	EchoPin[1]=Echo2;
	TrigPin[1]=Trig2;
}

#if defined DHT11_SENSOR
FSSensor::FSSensor(int Echo1, int Trig1, int Echo2, int Trig2, DHT *dht11) {
	temp = 20.0;
	timestamp=millis();
	distance = 0;
	pellets = 0;
	dht=dht11;
	EchoPin[0]=Echo1;
	TrigPin[0]=Trig1;
	EchoPin[1]=Echo2;
	TrigPin[1]=Trig2;
}
#endif
#endif

void FSSensor::begin() {
#if defined DHT11_SENSOR
	dht->begin();
#endif
#if defined HCSR04
	for( int i=0; i<=NUM_SENS-1; i++) {
		if ( TrigPin[i] != 1000 ) {
			pinMode(TrigPin[i], OUTPUT);
			digitalWrite(TrigPin[i], LOW);
		}
		if ( EchoPin[i] != 1000 ) {
			pinMode(EchoPin[i], INPUT);
		}
	}
#endif
#if defined SENS_VL53L0X
	pinMode(MUX_PIN, OUTPUT);
	pinMode(MUX_PIN2, OUTPUT);
	digitalWrite(MUX_PIN, LOW);
	digitalWrite(MUX_PIN2, LOW);
	Wire.begin();
	for ( int i=0; i<=NUM_SENS-1; i++) {
		SelectSensor(i);
		tof_sens[i]->setTimeout(500);
		tof_sens[i]->init(true);
		// lower the return signal rate limit (default is 0.25 MCPS)
  		tof_sens[i]->setSignalRateLimit(0.17);
		// increase laser pulse periods (defaults are 14 and 10 PCLKs)
		tof_sens[i]->setVcselPulsePeriod(VL53L0X::VcselPeriodPreRange, 18);
		tof_sens[i]->setVcselPulsePeriod(VL53L0X::VcselPeriodFinalRange, 14);
		tof_sens[i]->setMeasurementTimingBudget(200000);
	}
#endif
}

#if defined SENS_VL53L0X
void FSSensor::SelectSensor(int SenNr) {
	switch(SenNr) {
	case 0:
		digitalWrite(MUX_PIN, LOW);
		digitalWrite(MUX_PIN2, LOW);
		break;
	case 1:
		digitalWrite(MUX_PIN, HIGH);
		digitalWrite(MUX_PIN2, LOW);
		break;
	case 2:
		digitalWrite(MUX_PIN, LOW);
		digitalWrite(MUX_PIN2, HIGH);
		break;
	case 3:
		digitalWrite(MUX_PIN, HIGH);
		digitalWrite(MUX_PIN2, HIGH);
		break;
	}
	delay(100);
	yield();
}
	
#endif
void FSSensor::ReadSensor(int SenNr) {
	int ValidValues=0;

	if ( SenNr >= NUM_SENS ) return;
#if defined DHT11_SENSOR
	temp=dht->readTemperature();
	hum=dht->readHumidity();
	lagertemp= temp + TEMP_OFFSET;
#endif
	distance=0.0;
#if defined HCSR04
	//define the vars to hold the needed
	//data
	long duration[NR_MESSURE+1];
	long finalduration=0;
	float speed = 0.03436;
	float MeasuredDistance = 0;
	//Berechnung der Schallgeschwindigkeit in cm
	speed=(331.5*(sqrt(1+(lagertemp/273.15))))/100000;

	//Initialize the trigger pin with a clean LOW Signal
	digitalWrite(TrigPin[SenNr], LOW);
	delayMicroseconds(10);
	ValidValues=0;
	for (int messung=0; messung<=NR_MESSURE; messung++) {
		//Now give the TRIG PIN a defined level to 0v
		// digitalWrite(TrigPin[SenNr], LOW);
		//wait some time
		// delayMicroseconds(30);
		//and send out the TRIGGER (TRIG Pin to 3.3v)
		noInterrupts();
		digitalWrite(TrigPin[SenNr], HIGH);
		//to start the measurement the signal must be high
		//for at least 10 MicroSeconds. to be shure let us wait 30
		delayMicroseconds(10);
		//and set the pin again to LOW (=0v)
		digitalWrite(TrigPin[SenNr], LOW);
		//now start reading the ECHO PIN
		//wait for the PIN Getting High and back to LOW
		//and give us the time between HIGH -> and LOW
		duration[messung] = ( long ) pulseIn(EchoPin[SenNr], HIGH);
		if ( duration[messung] > 0 ) {
			finalduration = finalduration + duration[messung];
			ValidValues++;
		}
		interrupts();
		yield();
		//give sensor a moment for cooldown
		//spec say 50 messurements per second are possible,
		//so we will wait 20 ms with the meassurement time
		//we will get something below 50/sec.
		delay(30);
	}
	//Lass uns den Höchsten Wert Suchen
	if ( ValidValues > 0 ) {
		finalduration = finalduration / ValidValues;
		MeasuredDistance = finalduration * speed / 2;
		if ( ( MeasuredDistance < 2000 ) && ( MeasuredDistance > 0 ) ) {
			sens[SenNr]=MeasuredDistance;
		} else {
			sens[SenNr]=-1;
		}
	} else {
		sens[SenNr]=-2;
	}
	ValidValues=0;
	for ( int sensnr=0; sensnr<=NUM_SENS-1; sensnr++) {
		if ( sens[sensnr] > 0 ) {
			distance= distance + sens[sensnr];
			ValidValues++;
		}
	}
	if ( ValidValues == 0 ) {
		distance=-1;
	} else {
		distance=distance/ValidValues;
	}
#endif
#if defined SENS_VL53L0X
	float tmp_dist;
	SelectSensor(SenNr);
	distance=0;
	ValidValues=0;
	for ( int messung=1;messung<=NR_MESSURE; messung++) {
		tmp_dist=( tof_sens[SenNr]->readRangeSingleMillimeters() / 10 )/SensKorr[SenNr];
		if ( ! tof_sens[SenNr]->timeoutOccurred() ) {
			if ( tmp_dist <= 210 ) {
				distance=distance+tmp_dist;
				ValidValues++;
			}
		}
		delay(100);
	}
	yield();
	if ( ValidValues > 0 ) {
		sens[SenNr]=distance/ValidValues;
	} else {
		sens[SenNr]=-1;
	}
	for ( int sensnr=0; sensnr<=NUM_SENS-1; sensnr++) {
		if ( sens[sensnr] > 0 ) {
			distance=distance + sens[sensnr];
			ValidValues++;
		}
	}
	if ( ValidValues == 0 ) {
		distance=-1;
	} else {
		distance=distance/ValidValues;
	}
#endif
}

void FSSensor::Calc() {
       	float Hoehe; //stores the calculated value for the filling high of the pellets
       	float SchraegHoch; //stores the calculated value for the high of the WINKEL° area of the stock
       	float WannenVolumen; //stores the volume of the WINKEL* area of the stock
	/*int ValidValues=0;
	distance=0.0;
	for ( int sensnr=0; sensnr<=NUM_SENS-1; sensnr++) {
		if ( sens[sensnr] > 0 ) {
			distance=distance + sens[sensnr];
			ValidValues++;
		}
	}
	if ( ValidValues == 0 ) {
		distance=0;
	} else {
		distance=distance/ValidValues;
	}*/
	//Höhe ergibt sich aus der Raumhöhe abzüglich der Bodendicke (OSB Plattenstärke)
	//Die Füllhöhe des lagers ist dann die Lager Höhe abzüglich der gemessenen Distance
        Hoehe = KHOEHE - BODENHOEHE - (distance / 100);
	//Die Höhe eder Schrägen ergibt sich aus Lagerbreite und Winkel der Schrägen
	//Das läst sich mit den Formeln für Dreiecksberechnung ausrechnen
        SchraegHoch = SCHRAEGBREITE*tan(M_PI/180*WINKEL);
        if ( ( Hoehe - SchraegHoch ) > 0 ) {
		//Die Füllhöhe ist noch oberhalb der Schrägen
		//Also müssen wir das komplette Volumen der Schrägen
		//sowie die Füllhöhe ÜBER den Schrägen berechnen
                WannenVolumen = (SchraegHoch*SCHRAEGBREITE*LAENGE)+(WANNENBODEN*SchraegHoch*LAENGE);
                pellets = ((Hoehe-SchraegHoch)*LAENGE*BREITE) + WannenVolumen;
        } else {
                if ( Hoehe <= 0 ) {
			//OKAY - das Lager ist wohl leer
                        pellets = 0;
                } else {
			//Wir bruachen nur die Höhe in den Schrägen berechnen - damit
			//Stimmt aber unsere Breite nicht mehr und muss über die
			//Winkelfunktionen, sowie Pathagoras neu berechnet werden
                        pellets = ( Hoehe * (Hoehe/tan(M_PI/180*WINKEL)) * LAENGE * 2 ) + (WANNENBODEN * Hoehe * LAENGE);
                }
        }
	//Aktuell entspricht pellets dem Volumen
	//daraus müssen wir jetzt das Gewicht umrechnen
        pellets = pellets * PELLETSGEWICHT;
        timestamp = millis();
}

float FSSensor::GetDistance() {
	return(distance);
}
float FSSensor::GetPellets() {
	return(pellets);
}
float FSSensor::GetSens(int nr) {
	return(sens[nr]);
}
#if defined DHT11_SENSOR
float FSSensor::GetTemperature() {
	return(temp);
}
float FSSensor::GetHumidity() {
	return(hum);
}
#endif
unsigned long FSSensor::GetLastUpdate() {
	return(timestamp);
}
